//
//  FilePlayerController.m
//  PlayFileCocoa
//
//  Copyright 2011 JG Electronics. All rights reserved.
//	Remarks, reports to: www.jgelectronics.nl.
//
//	Derived from Apple's example program PlayFile and chapter 7 of the book Core Audio, authors Kevin Avila and Chris Adamson.
//	The application icon is from Vincent Garnier http://benjigarner.deviantart.com/art/Fruity-Apples-80805804
//
//	OSX10.6 and XCODE 4.0.1
//
/*
 Disclaimer: IMPORTANT: 
 This JG Electronics software is supplied to you by JG Electronics as FREE AND OPEN SOURCE SOFTWARE. 
 
 The JG Electronics Software is provided by JG Electronics on an "AS IS" basis.
 JG ELECTRONICS MAKES NO WARRANTIES, EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION
 THE IMPLIED WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS
 FOR A PARTICULAR PURPOSE, REGARDING THE JG ELECTRONICS SOFTWARE OR ITS USE AND
 OPERATION ALONE OR IN COMBINATION WITH YOUR PRODUCTS.
 
 IN NO EVENT SHALL JG ELECTRONICS BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL
 OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) ARISING IN ANY WAY OUT OF THE USE, REPRODUCTION,
 MODIFICATION AND/OR DISTRIBUTION OF THE JG ELECTRONICS SOFTWARE, HOWEVER CAUSED
 AND WHETHER UNDER THEORY OF CONTRACT, TORT (INCLUDING NEGLIGENCE),
 STRICT LIABILITY OR OTHERWISE, EVEN IF JG ELECTRONICS HAS BEEN ADVISED OF THE
 POSSIBILITY OF SUCH DAMAGE.
 */

#import "FilePlayerController.h"


@implementation FilePlayerController

	#pragma mark -
	#pragma mark === Accessor Methods ===
//	*********************************************************************************************************************************
//	archive strings
	@synthesize fpFileNameStr;

- (id)init
{
    self = [super init];
    if (self) {
	//	notifications
		NSNotificationCenter *	nc = [NSNotificationCenter defaultCenter];
		[nc addObserver:self selector:@selector(fpPlayBackEndNotification:) name:@"playbackEndNotification" object:nil];
        fpFilePlayererPtr = [[FilePlayerModel alloc] init];
	//
		fpFileNameStr = @"";
    }
    
    return self;
}

- (void)dealloc
{
	[fpFilePlayererPtr release];
    [super dealloc];
}

	#pragma mark -
	#pragma mark === Notification Methods ===
//	*********************************************************************************************************************************
- (void)fpPlayBackEndNotification:(NSNotification *)notification
{
	[fpBrowse setEnabled:YES];
}

	#pragma mark -
	#pragma mark === Action Methods ===
//	*********************************************************************************************************************************
- (IBAction)fpReadButtons:(id)sender
{
	NSLog(@"fpReadButtons Started");
	BOOL			succes;
	
	switch([sender tag])
	{
		case 0:		//	browse
			[self fpOpenSoundFile];
			break;
		case 1:		//	start to play file
			NSLog(@"fpFilePathStr: %@", fpFileNameStr);
			NSURL *	soundFileURL = [NSURL fileURLWithPath:fpFileNameStr];
			succes = [fpFilePlayererPtr fpConfigureAudioFile:(CFURLRef)soundFileURL];
			if(succes == NO) {
				NSLog(@"ConfigureAudioFile failed");
			}
			[fpFilePlayererPtr fpStartPlay];
			[fpStartPlayBack setEnabled:NO];
			break;
	}
}

#pragma mark -
#pragma mark === Data Archive Methods ===
//	*********************************************************************************************************************************
- (void)fpOpenSoundFile
{
    NSOpenPanel*	openPanel = [NSOpenPanel openPanel];
	NSArray *		fileTypes = [NSArray arrayWithObjects:@"aiff", @"mp3", nil];

	[openPanel setAllowedFileTypes:fileTypes];
    [openPanel setAllowsMultipleSelection:NO];
	[openPanel beginSheetModalForWindow:fpWindow completionHandler:^(NSInteger result) {
        if (result == NSOKButton) {
            [openPanel orderOut:self];		//	close panel before we might present an error
			[fpBrowse  setEnabled:NO];
			[fpStartPlayBack setEnabled:YES];
			NSArray *	filesToOpen = [openPanel filenames];
			[self setFpFileNameStr:[filesToOpen objectAtIndex:0]];
			NSLog(@"Open Panel Returned: %@.\n", fpFileNameStr);
        }
    }];
}

@end
